;;; org-block-hide --- hide src blocks in org-mode files

(defvar org-block-hide-src-block-regexp "#\\+begin_src \\(.+\\)[ \t]*\\([ \t]+\\([^\n]+\\)\\)?\n\\([^\000]+?\\)#\\+end_src")

(defun org-block-hide-ui-src-block-cycle-maybe ()
  (let ((case-fold-search t))
    (if (save-excursion
          (beginning-of-line 1)
          (looking-at org-block-hide-src-block-regexp))
        (progn (call-interactively 'org-block-hide-ui-src-block-cycle)
               t) ;; to signal that we took action
      nil))) ;; to signal that we did not

(defun org-block-hide-ui-src-block-cycle ()
  "Cycle the visibility of the current source code block"
  (interactive)
  ;; should really do this once in an (org-mode hook)
  (add-to-invisibility-spec '(org-block-hide-ui . t))
  (message "trying out source block")
  (save-excursion
    (beginning-of-line)
    (if (re-search-forward org-block-hide-src-block-regexp nil t)
        (let ((start (- (match-beginning 4) 1)) ;; beginning of body
              (end (match-end 0))) ;; end of entire body
          (if (memq t (mapcar (lambda (overlay)
                                (eq (overlay-get overlay 'invisible) 'org-block-hide-ui))
                              (overlays-at start)))
              (remove-overlays start end 'invisible 'org-block-hide-ui)
            (overlay-put (make-overlay start end) 'invisible 'org-block-hide-ui)))
      (error "not looking at a source block"))))

;; org-tab-after-check-for-cycling-hook
(add-hook 'org-tab-first-hook 'org-block-hide-ui-src-block-cycle-maybe)